import knex from 'knex';
declare abstract class Type<A> {
    readonly _A: A;
    withQuery(q: knex.QueryBuilder): Promise<A>;
    abstract getDefinition(): any;
    abstract getSelect(): string[];
}
declare type TypeOf<C extends Type<any>> = C['_A'];
export interface Props {
    [key: string]: Type<any>;
}
declare class ValueType<A> extends Type<A> {
    protected field: string;
    private option;
    static of<A>(field: string, option?: TransformOption): ValueType<A>;
    constructor(field: string, option: TransformOption);
    getDefinition(): {
        column: string;
        id: boolean;
    };
    getSelect(): string[];
}
declare class ArrayType<C extends Type<any>> extends Type<Array<TypeOf<C>>> {
    private value;
    constructor(value: C);
    getDefinition(): any[];
    getSelect(): string[];
}
declare class TypeType<P extends Props> extends Type<{
    [K in keyof P]: TypeOf<P[K]>;
}> {
    private value;
    constructor(value: P);
    getDefinition(): any;
    getSelect(): string[];
}
declare class NullableType<P extends Props> extends Type<{
    [K in keyof P]: TypeOf<P[K]>;
} | null> {
    private value;
    constructor(value: P);
    getDefinition(): any;
    getSelect(): string[];
}
interface TransformOption {
    id: boolean;
}
export declare const nullableType: <P extends Props>(v: P) => NullableType<P>;
export declare const type: <P extends Props>(v: P) => TypeType<P>;
export declare const number: (field: string, option?: TransformOption | undefined) => Type<number>;
export declare const string: (field: string, option?: TransformOption | undefined) => Type<string>;
export declare const date: (field: string, option?: TransformOption | undefined) => Type<Date>;
export declare const boolean: (field: string, option?: TransformOption | undefined) => Type<boolean>;
export declare const nullableNumber: (field: string) => ValueType<number | null>;
export declare const nullableString: (field: string) => ValueType<string | null>;
export declare const nullableDate: (field: string) => ValueType<Date | null>;
export declare const array: <C extends Type<any>>(v: C) => ArrayType<C>;
export {};
